@@ -48,8 +48,8 @@ class CacheExceptionLockingFailed(CacheException):
 class CacheExceptionDpkgInterrupted(CacheException):
     pass
 
-# the initrd space required in /boot for each kernel
-KERNEL_INITRD_SIZE = 12*1024*1024
+# the initrd/vmlinuz/abi space required in /boot for each kernel
+KERNEL_INITRD_SIZE = 19 * 1024 * 1024
 
 class FreeSpaceRequired(object):
     """ FreeSpaceRequired object:
@@ -154,6 +154,8 @@ class MyCache(apt.Cache):
         " init logging, create log file"
         logdir = self.config.getWithDefault("Files","LogDir",
                                             "/var/log/dist-upgrade")
+        if not os.path.exists(logdir):
+            os.makedirs(logdir)
         apt_pkg.Config.Set("Dir::Log",logdir)
         apt_pkg.Config.Set("Dir::Log::Terminal","apt-term.log")
         self.logfd = os.open(os.path.join(logdir,"apt.log"),
@@ -223,6 +225,10 @@ class MyCache(apt.Cache):
         self.unlockListsDir()
         res = apt.Cache.update(self, fprogress)
         self.lockListsDir()
+        if fprogress.release_file_download_error:
+            # FIXME: not ideal error message, but we just reuse a 
+            #        existing one here to avoid a new string
+            raise IOError(_("The server may be overloaded"))
         if res == False:
             raise IOError("apt.cache.update() returned False, but did not raise exception?!?")
 
@@ -259,6 +265,11 @@ class MyCache(apt.Cache):
             return False
         return ver.Downloadable
     
+    def pkgAutoRemovable(self, pkg):
+        """ check if the pkg is auto-removable """
+        return (pkg.isInstalled and 
+                self._depcache.IsGarbage(pkg._pkg))
+
     def fixBroken(self):
         """ try to fix broken dependencies on the system, may throw
             SystemError when it can't"""
@@ -343,6 +354,16 @@ class MyCache(apt.Cache):
             if not (self[pkg].markedInstall or self[pkg].markedUpgrade):
                 logging.error("Installing/upgrading '%s' failed" % pkg)
                 #raise (SystemError, "Installing '%s' failed" % pkg)
+                return False
+        return True
+    def markUpgrade(self, pkg, reason=""):
+        logging.debug("Upgrading '%s' (%s)" % (pkg, reason))
+        if self.has_key(pkg) and self[pkg].isInstalled:
+            self[pkg].markUpgrade()
+            if not self[pkg].markedUpgrade:
+                logging.error("Upgrading '%s' failed" % pkg)
+                return False
+        return True
     def markRemove(self, pkg, reason=""):
         logging.debug("Removing '%s' (%s)" % (pkg, reason))
         if self.has_key(pkg):
@@ -352,24 +373,24 @@ class MyCache(apt.Cache):
         if self.has_key(pkg):
             self._depcache.MarkDelete(self[pkg]._pkg,True)
 
+    def _keepInstalled(self, pkgname, reason):
+        if (self.has_key(pkgname)
+            and self[pkgname].isInstalled
+            and self[pkgname].markedDelete):
+            self.markInstall(pkgname, reason)
+
     def keepInstalledRule(self):
         """ run after the dist-upgrade to ensure that certain
             packages are kept installed """
-        def keepInstalled(self, pkgname, reason):
-            if (self.has_key(pkgname)
-                and self[pkgname].isInstalled
-                and self[pkgname].markedDelete):
-                self.markInstall(pkgname, reason)
-                
         # first the global list
         for pkgname in self.config.getlist("Distro","KeepInstalledPkgs"):
-            keepInstalled(self, pkgname, "Distro KeepInstalledPkgs rule")
+            self._keepInstalled(pkgname, "Distro KeepInstalledPkgs rule")
         # the the per-metapkg rules
         for key in self.metapkgs:
             if self.has_key(key) and (self[key].isInstalled or
                                       self[key].markedInstall):
                 for pkgname in self.config.getlist(key,"KeepInstalledPkgs"):
-                    keepInstalled(self, pkgname, "%s KeepInstalledPkgs rule" % key)
+                    self._keepInstalled(pkgname, "%s KeepInstalledPkgs rule" % key)
 
         # only enforce section if we have a network. Otherwise we run
         # into CD upgrade issues for installed language packs etc
@@ -378,20 +399,21 @@ class MyCache(apt.Cache):
             # now the KeepInstalledSection code
             for section in self.config.getlist("Distro","KeepInstalledSection"):
                 for pkg in self:
-                    if pkg.markedDelete and pkg.section == section:
-                        keepInstalled(self, pkg.name, "Distro KeepInstalledSection rule: %s" % section)
+                    if pkg.candidateDownloadable and pkg.markedDelete and pkg.section == section:
+                        self._keepInstalled(pkg.name, "Distro KeepInstalledSection rule: %s" % section)
             for key in self.metapkgs:
                 if self.has_key(key) and (self[key].isInstalled or
                                           self[key].markedInstall):
                     for section in self.config.getlist(key,"KeepInstalledSection"):
                         for pkg in self:
-                            if pkg.markedDelete and pkg.section == section:
-                                keepInstalled(self, pkg.name, "%s KeepInstalledSection rule: %s" % (key, section))
+                            if pkg.candidateDownloadable and pkg.markedDelete and pkg.section == section:
+                                self._keepInstalled(pkg.name, "%s KeepInstalledSection rule: %s" % (key, section))
         
 
     def postUpgradeRule(self):
         " run after the upgrade was done in the cache "
         for (rule, action) in [("Install", self.markInstall),
+                               ("Upgrade", self.markUpgrade),
                                ("Remove", self.markRemove),
                                ("Purge", self.markPurge)]:
             # first the global list
@@ -452,7 +474,8 @@ class MyCache(apt.Cache):
         try:
             # get new detection module and use the modalises files
             # from within the release-upgrader
-            nv = NvidiaDetection(datadir="./modaliases/")
+            nv = NvidiaDetection(datadir="./modaliases/",
+                                 obsolete="./nvidia-obsolete.pkgs")
             #nv = NvidiaDetection()
             # check if a binary driver is installed now
             for oldDriver in nv.oldPackages:
@@ -465,8 +488,10 @@ class MyCache(apt.Cache):
             # check which one to use
             driver = nv.selectDriver()
             logging.debug("nv.selectDriver() returned '%s'" % driver)
-            if (self.has_key(driver) and not
-                (self[driver].markedInstall or self[driver].markedUpgrade)):
+            if not self.has_key(driver):
+                logging.warning("no '%s' found" % driver)
+                return False
+            if not (self[driver].markedInstall or self[driver].markedUpgrade):
                 self[driver].markInstall()
                 logging.info("installing %s as suggested by NvidiaDetector" % driver)
                 return True
@@ -474,6 +499,43 @@ class MyCache(apt.Cache):
             logging.error("NvidiaDetection returned a error: %s" % e)
         return False
 
+
+    def getKernelsFromBaseInstaller(self):
+        """get the list of recommended kernels from base-installer"""
+        kernels = Popen(["/bin/sh", "./get_kernel_list.sh"],
+                        stdout=PIPE).communicate()[0]
+        kernels = filter(lambda x : len(x) > 0,
+                         map(string.strip, kernels.split("\n")))
+        logging.debug("./get_kernel_list.sh returns: %s" % kernels)
+        return kernels
+
+    def _selectKernelFromBaseInstaller(self):
+        """ use the get_kernel_list.sh script (that uses base-installer)
+            to figure out what kernel is most suitable for the system
+        """
+        # check if we have a kernel from that list installed first
+        kernels = self.getKernelsFromBaseInstaller()
+        for kernel in kernels:
+            if not self.has_key(kernel):
+                logging.debug("%s not available in cache" % kernel)
+                continue
+            # this can happen e.g. on cdrom -> cdrom only upgrades
+            # where on hardy we have linux-386 but on the lucid CD 
+            # we only have linux-generic
+            if not self[kernel].candidateDownloadable:
+                logging.debug("%s not downloadable" % kernel)
+                continue
+            # check if installed
+            if self[kernel].isInstalled or self[kernel].markedInstall:
+                logging.debug("%s kernel already installed" % kernel)
+                if self[kernel].isUpgradable and not self[kernel].markedUpgrade:
+                    self.markUpgrade(kernel, "Upgrading kernel from base-installer")
+                return 
+        # if we have not found a kernel yet, use the first one that installs
+        for kernel in kernels:
+            if self.markInstall(kernel, "Selecting new kernel from base-installer"):
+                return
+
     def checkForKernel(self):
         """ check for the running kernel and try to ensure that we have
             an updated version
@@ -488,14 +550,11 @@ class MyCache(apt.Cache):
         dmesg = Popen(["dmesg"],stdout=PIPE).communicate()[0]
         if "WARNING: NR_CPUS limit" in dmesg:
             logging.debug("UP kernel on SMP system!?!")
-            flavour = "generic"
-        kernel = "linux-image-%s" % flavour
-        if not self.has_key(kernel):
-            logging.warning("No kernel: '%s'" % kernel)
-            return False
-        if not (self[kernel].isInstalled or self[kernel].markedInstall):
-            logging.debug("Selecting new kernel '%s'" % kernel)
-            self[kernel].markInstall()
+        # use base-installer to get the kernel we want (if it exists)
+        if os.path.exists("./get_kernel_list.sh"):
+            self._selectKernelFromBaseInstaller()
+        else:
+            logging.debug("skipping ./get_kernel_list.sh: not found")
         return True
 
     def checkPriority(self):
@@ -520,7 +579,7 @@ class MyCache(apt.Cache):
     def updateGUI(self, view, lock):
         while lock.locked():
             view.processEvents()
-            time.sleep(0.01)
+            time.sleep(0.03)
 
     @withResolverLog
     def distUpgrade(self, view, serverMode, partialUpgrade):
@@ -549,6 +608,7 @@ class MyCache(apt.Cache):
             self.postUpgradeRule()
 
             # install missing meta-packages (if not in server upgrade mode)
+            self._keepBaseMetaPkgsInstalled(view)
             if not serverMode:
                 # if this fails, a system error is raised
                 self._installMetaPkgs(view)
@@ -665,7 +725,25 @@ class MyCache(apt.Cache):
                 self[pkgname].candidateVersion == ver and
                 (self[pkgname].markedInstall or
                  self[pkgname].markedUpgrade)):
-                raise SystemError, "Trying to install blacklisted version '%s'" % bv
+                raise SystemError, _("Trying to install blacklisted version '%s'") % bv
+        return True
+    
+    def _lookupPkgRecord(self, pkg):
+        """ 
+        helper to make sure that the pkg._records is pointing to the right
+        location - needed because python-apt 0.7.9 dropped the python-apt
+        version but we can not yet use the new version because on upgrade
+        the old version is still installed
+        """ 
+        ver = pkg._pcache._depcache.GetCandidateVer(pkg._pkg)
+        if ver is None:
+            print "No candidate ver: ", pkg.name
+            return False
+        if ver.FileList is None:
+            print "No FileList for: %s " % self._pkg.Name()
+            return False
+        f, index = ver.FileList.pop(0)
+        pkg._pcache._records.Lookup((f, index))
         return True
 
     @property
@@ -673,7 +751,7 @@ class MyCache(apt.Cache):
         tasks = {}
         installed_tasks = set()
         for pkg in self:
-            if not pkg._lookupRecord():
+            if not self._lookupPkgRecord(pkg):
                 logging.debug("no PkgRecord found for '%s', skipping " % pkg.name)
                 continue
             for line in pkg._pcache._records.Record.split("\n"):
@@ -698,7 +776,7 @@ class MyCache(apt.Cache):
         for pkg in self:
             if pkg.markedInstall or pkg.isInstalled:
                 continue
-            pkg._lookupRecord()
+            self._lookupPkgRecord(pkg)
             if not (hasattr(pkg._pcache._records,"Record") and pkg._pcache._records.Record):
                 logging.warning("can not find Record for '%s'" % pkg.name)
                 continue
@@ -710,6 +788,10 @@ class MyCache(apt.Cache):
                             pkg.markInstall()
         return True
     
+    def _keepBaseMetaPkgsInstalled(self, view):
+        for pkg in self.config.getlist("Distro","BaseMetaPkgs"):
+            self._keepInstalled(pkg, "base meta package keep installed rule")
+
     def _installMetaPkgs(self, view):
 
         def metaPkgInstalled():
@@ -763,6 +845,7 @@ class MyCache(apt.Cache):
                                      "this as a bug. "))
                         return False
                     logging.debug("markedInstall: '%s' -> '%s'" % (key, self[key].markedInstall))
+                    break
         # check if we actually found one
         if not metaPkgInstalled():
             # FIXME: provide a list
@@ -781,6 +864,7 @@ class MyCache(apt.Cache):
     def _inRemovalBlacklist(self, pkgname):
         for expr in self.removal_blacklist:
             if re.compile(expr).match(pkgname):
+                logging.debug("blacklist expr '%s' matches '%s'" % (expr, pkgname))
                 return True
         return False
 
@@ -860,6 +944,29 @@ class MyCache(apt.Cache):
                 unused_dependencies.add(pkg.name)
         return unused_dependencies
 
+    def get_installed_demoted_packages(self):
+        """ return list of installed and demoted packages 
+
+            If a demoted package is a automatic install it will be skipped
+        """
+        demotions = set()
+        demotions_file = self.config.get("Distro","Demotions")
+        if os.path.exists(demotions_file):
+            map(lambda pkgname: demotions.add(pkgname.strip()),
+                filter(lambda line: not line.startswith("#"),
+                       open(demotions_file).readlines()))
+        installed_demotions = set()
+        for demoted_pkgname in demotions:
+            if not self.has_key(demoted_pkgname):
+                continue
+            pkg = self[demoted_pkgname]
+            if (not pkg.isInstalled or
+                self._depcache.IsAutoInstalled(pkg._pkg) or
+                pkg.markedDelete):
+                continue
+            installed_demotions.add(pkg.name)
+        return list(installed_demotions)
+
     def _getForeignPkgs(self, allowed_origin, fromDist, toDist):
         """ get all packages that are installed from a foreign repo
             (and are actually downloadable)
@@ -933,11 +1040,15 @@ class MyCache(apt.Cache):
             if not os.path.exists(aufs_rw_dir):
                 os.makedirs(aufs_rw_dir)
         logging.debug("cache aufs_rw_dir: %s" % aufs_rw_dir)
-        for d in ["/","/usr","/var","/boot", archivedir, aufs_rw_dir, "/home"]:
+        for d in ["/","/usr","/var","/boot", archivedir, aufs_rw_dir, "/home","/tmp/"]:
             d = os.path.realpath(d)
             fs_id = make_fs_id(d)
-            st = os.statvfs(d)
-            free = st[statvfs.F_BAVAIL]*st[statvfs.F_FRSIZE]
+            if os.path.exists(d):
+                st = os.statvfs(d)
+                free = st[statvfs.F_BAVAIL]*st[statvfs.F_FRSIZE]
+            else:
+                logging.warn("directory '%s' does not exists" % d)
+                free = 0
             if fs_id in mnt_map:
                 logging.debug("Dir %s mounted on %s" % (d,mnt_map[fs_id]))
                 fs_free[d] = fs_free[mnt_map[fs_id]]
@@ -986,6 +1097,7 @@ class MyCache(apt.Cache):
                             ("/usr", self.additionalRequiredSpace),
                             ("/usr", 50*1024*1024),
                             ("/boot", space_in_boot), 
+                            ("/tmp", 5*1024*1024),   # /tmp for dkms LP: #427035
                             ("/", 10*1024*1024),     # small safety buffer /
                             (aufs_rw_dir, required_for_aufs),
                            ]:
