@@ -23,11 +23,11 @@ import pygtk
 pygtk.require('2.0')
 import gtk
 import gtk.gdk
-import gtk.glade
 import vte
 import gobject
 import pango
 import sys
+import locale
 import logging
 import time
 import subprocess
@@ -38,16 +38,17 @@ import os
 
 from DistUpgradeApport import *
 
-from DistUpgradeView import DistUpgradeView, FuzzyTimeToStr, estimatedDownloadTime, InstallProgress
-from UpdateManager.Common.SimpleGladeApp import SimpleGladeApp, bindtextdomain
+from DistUpgradeView import DistUpgradeView, FuzzyTimeToStr, InstallProgress, FetchProgress
+from SimpleGtkbuilderApp import SimpleGtkbuilderApp
 
 import gettext
-from gettext import gettext as _
+from DistUpgradeGettext import gettext as _
+
+gobject.threads_init()
 
 def utf8(str):
   return unicode(str, 'latin1').encode('utf-8')
 
-
 class GtkCdromProgressAdapter(apt.progress.CdromProgress):
     """ Report the cdrom add progress
         Subclass this class to implement cdrom add progress reporting
@@ -87,11 +88,12 @@ class GtkOpProgress(apt.progress.OpProgress):
       self.progressbar.set_text(" ")
 
 
-class GtkFetchProgressAdapter(apt.progress.FetchProgress):
+class GtkFetchProgressAdapter(FetchProgress):
     # FIXME: we really should have some sort of "we are at step"
     # xy in the gui
     # FIXME2: we need to thing about mediaCheck here too
     def __init__(self, parent):
+        FetchProgress.__init__(self)
         # if this is set to false the download will cancel
         self.status = parent.label_status
         self.progress = parent.progressbar_cache
@@ -111,12 +113,14 @@ class GtkFetchProgressAdapter(apt.progress.FetchProgress):
                                    buttons=gtk.BUTTONS_OK_CANCEL)
         dialog.set_markup(msg)
         res = dialog.run()
+        dialog.set_title("")
         dialog.destroy()
         if res == gtk.RESPONSE_OK:
             return True
         return False
     def start(self):
         #logging.debug("start")
+        FetchProgress.start(self)
         self.progress.set_fraction(0)
         self.status.show()
         self.button_cancel.show()
@@ -128,14 +132,14 @@ class GtkFetchProgressAdapter(apt.progress.FetchProgress):
     def pulse(self):
         # FIXME: move the status_str and progress_str into python-apt
         # (python-apt need i18n first for this)
-        apt.progress.FetchProgress.pulse(self)
+        FetchProgress.pulse(self)
         self.progress.set_fraction(self.percent/100.0)
         currentItem = self.currentItems + 1
         if currentItem > self.totalItems:
             currentItem = self.totalItems
 
         if self.currentCPS > 0:
-            self.status.set_text(_("Fetching file %li of %li at %sb/s") % (currentItem, self.totalItems, apt_pkg.SizeToStr(self.currentCPS)))
+            self.status.set_text(_("Fetching file %li of %li at %sB/s") % (currentItem, self.totalItems, apt_pkg.SizeToStr(self.currentCPS)))
             self.progress.set_text(_("About %s remaining") % FuzzyTimeToStr(self.eta))
         else:
             self.status.set_text(_("Fetching file %li of %li") % (currentItem, self.totalItems))
@@ -162,7 +166,6 @@ class GtkInstallProgressAdapter(InstallProgress):
         reaper = vte.reaper_get()
         reaper.connect("child-exited", self.child_exited)
         # some options for dpkg to make it die less easily
-        apt_pkg.Config.Set("DPkg::Options::","--force-overwrite")
         apt_pkg.Config.Set("DPkg::StopOnError","False")
 
     def startUpdate(self):
@@ -177,15 +180,16 @@ class GtkInstallProgressAdapter(InstallProgress):
         self.expander.set_sensitive(True)
         self.term.show()
         # if no libgnome2-perl is installed show the terminal
-        frontend="gnome"
-        if self._cache:
+        frontend= os.environ.get("DEBIAN_FRONTEND") or "gnome"
+        if frontend == "gnome" and self._cache:
           if (not self._cache.has_key("libgnome2-perl") or 
               not self._cache["libgnome2-perl"].isInstalled):
             frontend = "dialog"
             self.expander.set_expanded(True)
         self.env = ["VTE_PTY_KEEP_FD=%s"% self.writefd,
-                    "DEBIAN_FRONTEND=%s" % frontend,
                     "APT_LISTCHANGES_FRONTEND=none"]
+        if not os.environ.has_key("DEBIAN_FRONTEND"):
+          self.env.append("DEBIAN_FRONTEND=%s" % frontend)
         # do a bit of time-keeping
         self.start_time = 0.0
         self.time_ui = 0.0
@@ -194,14 +198,19 @@ class GtkInstallProgressAdapter(InstallProgress):
     def error(self, pkg, errormsg):
         InstallProgress.error(self, pkg, errormsg)
         logging.error("got an error from dpkg for pkg: '%s': '%s'" % (pkg, errormsg))
+	# we do not report followup errors from earlier failures
+        if gettext.dgettext('dpkg', "dependency problems - leaving unconfigured") in errormsg:
+	  return False
+
         #self.expander_terminal.set_expanded(True)
         self.parent.dialog_error.set_transient_for(self.parent.window_main)
         summary = _("Could not install '%s'") % pkg
-        msg = _("The upgrade will continue but the '%s' package may be "
-                "in a not working state. Please consider submitting a "
-                "bugreport about it.") % pkg
+        msg = _("The upgrade will continue but the '%s' package may not "
+                "be in a working state. Please consider submitting a "
+                "bug report about it.") % pkg
         markup="<big><b>%s</b></big>\n\n%s" % (summary, msg)
         self.parent.dialog_error.realize()
+        self.parent.dialog_error.set_title("")
         self.parent.dialog_error.window.set_functions(gtk.gdk.FUNC_MOVE)
         self.parent.label_error.set_markup(markup)
         self.parent.textview_error.get_buffer().set_text(utf8(errormsg))
@@ -219,6 +228,7 @@ class GtkInstallProgressAdapter(InstallProgress):
                 "a newer version.")
         markup = "<span weight=\"bold\" size=\"larger\">%s </span> \n\n%s" % (prim, sec)
         self.parent.label_conffile.set_markup(markup)
+        self.parent.dialog_conffile.set_title("")
         self.parent.dialog_conffile.set_transient_for(self.parent.window_main)
 
         # now get the diff
@@ -270,9 +280,14 @@ class GtkInstallProgressAdapter(InstallProgress):
             self.progress.set_text(_("About %s remaining") % FuzzyTimeToStr(eta))
           else:
             self.progress.set_text(" ")
+          # 2 == WEBKIT_LOAD_FINISHED - the enums is not exposed via python
+          if (self.parent._webkit_view and
+              self.parent._webkit_view.get_property("load-status") == 2):
+            self.parent._webkit_view.execute_script('progress("%s")' % percent)
 
     def child_exited(self, term, pid, status):
-        self.apt_status = os.WEXITSTATUS(status)
+        # we need to capture the full status here (not only the WEXITSTATUS)
+        self.apt_status = status
         self.finished = True
 
     def waitChild(self):
@@ -284,12 +299,7 @@ class GtkInstallProgressAdapter(InstallProgress):
         self.label_status.set_text("")
     
     def updateInterface(self):
-        try:
-          InstallProgress.updateInterface(self)
-        except ValueError, e:
-          logging.error("got ValueError from InstallPrgoress.updateInterface. Line was '%s' (%s)" % (self.read, e))
-          # reset self.read so that it can continue reading and does not loop
-	  self.read = ""
+        InstallProgress.updateInterface(self)
         # check if we haven't started yet with packages, pulse then
         if self.start_time == 0.0:
           self.progress.pulse()
@@ -303,7 +313,7 @@ class GtkInstallProgressAdapter(InstallProgress):
           self.parent.expander_terminal.set_expanded(True)
         while gtk.events_pending():
             gtk.main_iteration()
-	time.sleep(0.02)
+	time.sleep(0.005)
 
 class DistUpgradeVteTerminal(object):
   def __init__(self, parent, term):
@@ -318,15 +328,33 @@ class DistUpgradeVteTerminal(object):
     self.parent.expander_terminal.set_sensitive(True)
     if hidden==False:
       self.parent.expander_terminal.set_expanded(True)
-    self.term.fork_command(command=cmd[0],argv=cmd)
     self.finished = False
+    pid = self.term.fork_command(command=cmd[0],argv=cmd)
+    if pid < 0:
+      # error
+      return 
     while not self.finished:
       while gtk.events_pending():
         gtk.main_iteration()
       time.sleep(0.1)
     del self.finished
 
-class DistUpgradeViewGtk(DistUpgradeView,SimpleGladeApp):
+class HtmlView(object):
+  def __init__(self, webkit_view):
+    self._webkit_view = webkit_view
+  def open(self, url):
+    if not self._webkit_view:
+      return
+    self._webkit_view.open(url)
+    self._webkit_view.connect("load-finished", self._on_load_finished)
+  def show(self):
+    self._webkit_view.show()
+  def hide(self):
+    self._webkit_view.hide()
+  def _on_load_finished(self, view, frame):
+    view.show()
+
+class DistUpgradeViewGtk(DistUpgradeView,SimpleGtkbuilderApp):
     " gtk frontend of the distUpgrade tool "
     def __init__(self, datadir=None, logdir=None):
         self.logdir = logdir
@@ -334,12 +362,14 @@ class DistUpgradeViewGtk(DistUpgradeView,SimpleGladeApp):
           localedir=os.path.join(os.getcwd(),"mo")
           gladedir=os.getcwd()
         else:
-          localedir="/usr/share/locale/update-manager"
+          localedir="/usr/share/locale/"
           gladedir=os.path.join(datadir, "glade")
 
-        # FIXME: i18n must be somewhere relative do this dir
+	# check if we have a display etc
+	gtk.init_check()
+
         try:
-          bindtextdomain("update-manager", localedir)
+          locale.bindtextdomain("update-manager",localedir)
           gettext.textdomain("update-manager")
         except Exception, e:
           logging.warning("Error setting locales (%s)" % e)
@@ -350,10 +380,14 @@ class DistUpgradeViewGtk(DistUpgradeView,SimpleGladeApp):
         except gobject.GError, e:
           logging.debug("error setting default icon, ignoring (%s)" % e)
           pass
-        SimpleGladeApp.__init__(self, gladedir+"/DistUpgrade.glade",
-                                None, domain="update-manager")
+        SimpleGtkbuilderApp.__init__(self, 
+                                     gladedir+"/DistUpgrade.ui", 
+                                     "update-manager")
+        # terminal stuff
+        self.create_terminal()
+
         self.prev_step = 0 # keep a record of the latest step
-        # we dont use this currently
+        # we don't use this currently
         #self.window_main.set_keep_above(True)
         self.icontheme = gtk.icon_theme_get_default()
         # we keep a reference pngloader around so that its in memory
@@ -362,10 +396,17 @@ class DistUpgradeViewGtk(DistUpgradeView,SimpleGladeApp):
         self.pngloader = gtk.gdk.PixbufLoader("png")
         try:
           self.svgloader = gtk.gdk.PixbufLoader("svg")
+          self.svgloader.close()
         except gobject.GError, e:
-          logging.warning("svg pixbuf loader failed (%s)" % e)
+          logging.debug("svg pixbuf loader failed (%s)" % e)
           pass
-        
+        try:
+          import webkit
+          self._webkit_view = webkit.WebView()
+          self.vbox_main.pack_end(self._webkit_view)
+        except:
+          logging.exception("html widget")
+          self._webkit_view = None
         self.window_main.realize()
         self.window_main.window.set_functions(gtk.gdk.FUNC_MOVE)
         self._opCacheProgress = GtkOpProgress(self.progressbar_cache)
@@ -380,13 +421,10 @@ class DistUpgradeViewGtk(DistUpgradeView,SimpleGladeApp):
         column.add_attribute(render, "markup", 0)
         self.treeview_details.append_column(column)
         self.treeview_details.set_model(self.details_list)
-        self.vscrollbar_terminal.set_adjustment(self._term.get_adjustment())
-        # work around bug in VteTerminal here
-        self._term.realize()
-
         # Use italic style in the status labels
         attrlist=pango.AttrList()
-        attr = pango.AttrStyle(pango.STYLE_ITALIC, 0, -1)
+        #attr = pango.AttrStyle(pango.STYLE_ITALIC, 0, -1)
+        attr = pango.AttrScale(pango.SCALE_SMALL, 0, -1)
         attrlist.insert(attr)
         self.label_status.set_property("attributes", attrlist)
         # reasonable fault handler
@@ -404,11 +442,11 @@ class DistUpgradeViewGtk(DistUpgradeView,SimpleGladeApp):
       # so we still show a error message here
       apport_crash(type, value, tb)
       if not run_apport():
-        self.error(_("A fatal error occured"),
-                   _("Please report this as a bug (if you haven't already) and include the"
+        self.error(_("A fatal error occurred"),
+                   _("Please report this as a bug (if you haven't already) and include the "
                      "files /var/log/dist-upgrade/main.log and "
                      "/var/log/dist-upgrade/apt.log "
-                     "in your report. The upgrade aborts now.\n"
+                     "in your report. The upgrade is now aborted.\n"
                      "Your original sources.list was saved in "
                      "/etc/apt/sources.list.distUpgrade."),
                    "\n".join(lines))
@@ -416,13 +454,34 @@ class DistUpgradeViewGtk(DistUpgradeView,SimpleGladeApp):
 
     def getTerminal(self):
         return DistUpgradeVteTerminal(self, self._term)
+    def getHtmlView(self):
+        return HtmlView(self._webkit_view)
 
-    def create_terminal(self, arg1,arg2,arg3,arg4):
+    def _key_press_handler(self, widget, keyev):
+      # user pressed ctrl-c
+      if len(keyev.string) == 1 and ord(keyev.string) == 3:
+        summary = _("Ctrl-c pressed")
+        msg = _("This will abort the operation and may leave the system "
+                "in a broken state. Are you sure you want to do that?")
+        res = self.askYesNoQuestion(summary, msg)
+        logging.warning("ctrl-c press detected, user decided to pass it "
+                        "on: %s", res)        
+        return not res
+      return False
+
+    def create_terminal(self):
         " helper to create a vte terminal "
         self._term = vte.Terminal()
+        self._term.connect("key-press-event", self._key_press_handler)
         self._term.set_font_from_string("monospace 10")
         self._term.connect("contents-changed", self._term_content_changed)
         self._terminal_lines = []
+        self.hbox_custom.pack_start(self._term)
+        self._term.realize()
+        self.vscrollbar_terminal = gtk.VScrollbar()
+        self.vscrollbar_terminal.show()
+        self.hbox_custom.pack_start(self.vscrollbar_terminal)
+        self.vscrollbar_terminal.set_adjustment(self._term.get_adjustment())
         try:
           self._terminal_log = open(os.path.join(self.logdir,"term.log"),"w")
         except Exception, e:
@@ -441,7 +500,10 @@ class DistUpgradeViewGtk(DistUpgradeView,SimpleGladeApp):
           new_lines.append(line)
           if not line in self._terminal_lines:
             self._terminal_log.write(line+"\n")
-            self._terminal_log.flush()
+            try:
+              self._terminal_log.flush()
+            except IOError, e:
+              logging.exception("flush()")
         self._terminal_lines = new_lines
     def getFetchProgress(self):
         return self._fetchProgress
@@ -457,8 +519,14 @@ class DistUpgradeViewGtk(DistUpgradeView,SimpleGladeApp):
     def hideStep(self, step):
         image = getattr(self,"image_step%i" % step)
         label = getattr(self,"label_step%i" % step)
+        arrow = getattr(self,"arrow_step%i" % step)
         image.hide()
         label.hide()
+    def showStep(self, step):
+        image = getattr(self,"image_step%i" % step)
+        label = getattr(self,"label_step%i" % step)
+        image.show()
+        label.show()
     def abort(self):
         size = gtk.ICON_SIZE_MENU
         step = self.prev_step
@@ -487,6 +555,9 @@ class DistUpgradeViewGtk(DistUpgradeView,SimpleGladeApp):
         image = getattr(self,"image_step%i" % step)
         label = getattr(self,"label_step%i" % step)
         arrow = getattr(self,"arrow_step%i" % step)
+        # check if that step was not hidden with hideStep()
+        if not label.get_property("visible"):
+          return
         arrow.show()
         image.hide()
         attr = pango.AttrWeight(pango.WEIGHT_BOLD, 0, -1)
@@ -494,6 +565,7 @@ class DistUpgradeViewGtk(DistUpgradeView,SimpleGladeApp):
         label.set_property("attributes",attrlist)
 
     def information(self, summary, msg, extended_msg=None):
+      self.dialog_information.set_title("")
       self.dialog_information.set_transient_for(self.window_main)
       msg = "<big><b>%s</b></big>\n\n%s" % (summary,msg)
       self.label_information.set_markup(msg)
@@ -511,6 +583,7 @@ class DistUpgradeViewGtk(DistUpgradeView,SimpleGladeApp):
         gtk.main_iteration()
 
     def error(self, summary, msg, extended_msg=None):
+        self.dialog_error.set_title("")
         self.dialog_error.set_transient_for(self.window_main)
         #self.expander_terminal.set_expanded(True)
         msg="<big><b>%s</b></big>\n\n%s" % (summary, msg)
@@ -527,60 +600,16 @@ class DistUpgradeViewGtk(DistUpgradeView,SimpleGladeApp):
         self.dialog_error.hide()
         return False
 
-    def confirmChanges(self, summary, changes, downloadSize, actions=None):
+    def confirmChanges(self, summary, changes, downloadSize, 
+                       actions=None, removal_bold=True):
         # FIXME: add a whitelist here for packages that we expect to be
         # removed (how to calc this automatically?)
-        DistUpgradeView.confirmChanges(self, summary, changes, downloadSize)
-        pkgs_downgrade = len(self.toDowngrade)
-        pkgs_remove = len(self.toRemove)
-        pkgs_inst = len(self.toInstall)
-        pkgs_upgrade = len(self.toUpgrade)
-        msg = ""
-
-        if pkgs_downgrade > 0:
-            msg += gettext.ngettext("%d package is going to be downgraded.",
-                                    "%d packages are going to be downgraded.",
-                                    pkgs_downgrade) % pkgs_downgrade
-
-        if pkgs_remove > 0:
-            # FIXME: make those two seperate lines to make it clear
-            #        that the "%" applies to the result of ngettext
-            msg += gettext.ngettext("%d package is going to be removed.",
-                                    "%d packages are going to be removed.",
-                                    pkgs_remove) % pkgs_remove
-            msg += " "
-        if pkgs_inst > 0:
-            msg += gettext.ngettext("%d new package is going to be "
-                                    "installed.",
-                                    "%d new packages are going to be "
-                                    "installed.",pkgs_inst) % pkgs_inst
-            msg += " "
-        if pkgs_upgrade > 0:
-            msg += gettext.ngettext("%d package is going to be upgraded.",
-                                    "%d packages are going to be upgraded.",
-                                    pkgs_upgrade) % pkgs_upgrade
-            msg +=" "
-        if downloadSize > 0:
-            msg += _("\n\nYou have to download a total of %s. ") %\
-                     apt_pkg.SizeToStr(downloadSize)
-            msg += estimatedDownloadTime(downloadSize)
-            msg += "."
-
-        if (pkgs_upgrade + pkgs_inst + pkgs_remove) > 100:
-            msg += "\n\n%s" % _("Fetching and installing the upgrade can take several hours and "\
-                                "cannot be canceled at any time later.")
-
-        msg += "\n\n<b>%s</b>" % _("To prevent data loss close all open "\
-                                   "applications and documents.")
-
-        # Show an error if no actions are planned
-        if (pkgs_upgrade + pkgs_inst + pkgs_remove) < 1:
-            # FIXME: this should go into DistUpgradeController
-            summary = _("Your system is up-to-date")
-            msg = _("There are no upgrades available for your system. "
-                    "The upgrade will now be canceled.")
-            self.error(summary, msg)
-            return False
+        if not DistUpgradeView.confirmChanges(self, summary, changes, downloadSize):
+          return False
+        # append warning
+        self.confirmChangesMessage +=  "\n\n<b>%s</b>" %  \
+            _("To prevent data loss close all open "
+              "applications and documents.")
 
         if actions != None:
             self.button_cancel_changes.set_use_stock(False)
@@ -589,18 +618,25 @@ class DistUpgradeViewGtk(DistUpgradeView,SimpleGladeApp):
             self.button_confirm_changes.set_label(actions[1])
 
         self.label_summary.set_markup("<big><b>%s</b></big>" % summary)
-        self.label_changes.set_markup(msg)
+        self.label_changes.set_markup(self.confirmChangesMessage)
         # fill in the details
         self.details_list.clear()
         for dg in self.toDowngrade:
             self.details_list.append([_("<b>Downgrade %s</b>") % dg])
         for rm in self.toRemove:
-            self.details_list.append([_("<b>Remove %s</b>") % rm])
+            s = _("Remove %s") % rm
+            if removal_bold:
+              s = "<b>%s</b>" % s
+            self.details_list.append([s])
+        for rm in self.toRemoveAuto:
+            s = _("Remove (was auto installed) %s") % rm
+            self.details_list.append([s])
         for inst in self.toInstall:
             self.details_list.append([_("Install %s") % inst])
         for up in self.toUpgrade:
             self.details_list.append([_("Upgrade %s") % up])
         self.dialog_changes.set_transient_for(self.window_main)
+        self.dialog_changes.set_title("")
         # work around problem that scroll_to() does not 
         # work when it is not realized
         self.dialog_changes.realize()
@@ -620,6 +656,7 @@ class DistUpgradeViewGtk(DistUpgradeView,SimpleGladeApp):
                                    flags=gtk.DIALOG_MODAL,
                                    type=gtk.MESSAGE_QUESTION,
                                    buttons=gtk.BUTTONS_YES_NO)
+        dialog.set_title("")
         if default == 'No':
           dialog.set_default_response(gtk.RESPONSE_NO)
         else:
@@ -633,6 +670,7 @@ class DistUpgradeViewGtk(DistUpgradeView,SimpleGladeApp):
     
     def confirmRestart(self):
         self.dialog_restart.set_transient_for(self.window_main)
+        self.dialog_restart.set_title("")
         self.dialog_restart.realize()
         self.dialog_restart.window.set_functions(gtk.gdk.FUNC_MOVE)
         res = self.dialog_restart.run()
@@ -647,6 +685,7 @@ class DistUpgradeViewGtk(DistUpgradeView,SimpleGladeApp):
 
     def on_window_main_delete_event(self, widget, event):
         self.dialog_cancel.set_transient_for(self.window_main)
+        self.dialog_cancel.set_title("")
         self.dialog_cancel.realize()
         self.dialog_cancel.window.set_functions(gtk.gdk.FUNC_MOVE)
         res = self.dialog_cancel.run()
