@@ -72,6 +72,7 @@ from Core.utils import *
 from Core.UpdateList import UpdateList
 from Core.MyCache import MyCache, NotEnoughFreeSpaceError
 from Core.MetaRelease import Dist
+from SafeGConfClient import SafeGConfClient
 
 from DistUpgradeFetcher import DistUpgradeFetcherGtk
 from ChangelogViewer import ChangelogViewer
@@ -85,11 +86,13 @@ from MetaReleaseGObject import MetaRelease
 # - kill "all_changes" and move the changes into the "Update" class
 
 # list constants
-(LIST_CONTENTS, LIST_NAME, LIST_PKG, LIST_ORIGIN) = range(4)
+(LIST_CONTENTS, LIST_NAME, LIST_PKG, LIST_ORIGIN, LIST_TOGGLE_CHECKED) = range(5)
 
 # actions for "invoke_manager"
 (INSTALL, UPDATE) = range(2)
 
+# file that signals if we need to reboot
+REBOOT_REQUIRED_FILE = "/var/run/reboot-required"
 
 class UpdateManagerDbusControler(dbus.service.Object):
     """ this is a helper to provide the UpdateManagerIFace """
@@ -108,7 +111,6 @@ class UpdateManager(SimpleGtkbuilderApp):
   def __init__(self, datadir, options):
     self.setupDbus()
     gtk.window_set_default_icon_name("update-manager")
-
     self.datadir = datadir
     SimpleGtkbuilderApp.__init__(self, datadir+"glade/UpdateManager.ui",
                                  "update-manager")
@@ -141,9 +143,10 @@ class UpdateManager(SimpleGtkbuilderApp):
 
     # the treeview (move into it's own code!)
     self.store = gtk.ListStore(str, str, gobject.TYPE_PYOBJECT, 
-                               gobject.TYPE_PYOBJECT)
+                               gobject.TYPE_PYOBJECT, bool)
     self.treeview_update.set_model(self.store)
     self.treeview_update.set_headers_clickable(True);
+    self.treeview_update.set_direction(gtk.TEXT_DIR_LTR)
 
     tr = gtk.CellRendererText()
     tr.set_property("xpad", 6)
@@ -153,7 +156,7 @@ class UpdateManager(SimpleGtkbuilderApp):
     cr.set_property("xpad", 6)
     cr.connect("toggled", self.toggled)
 
-    column_install = gtk.TreeViewColumn("Install", cr)
+    column_install = gtk.TreeViewColumn("Install", cr, active=LIST_TOGGLE_CHECKED)
     column_install.set_cell_data_func (cr, self.install_column_view_func)
     column = gtk.TreeViewColumn("Name", tr, markup=LIST_CONTENTS)
     column.set_resizable(True)
@@ -181,18 +184,13 @@ class UpdateManager(SimpleGtkbuilderApp):
     if not os.path.exists("/usr/bin/software-properties-gtk"):
         self.button_settings.set_sensitive(False)
 
-    self.gconfclient = gconf.client_get_default()
+    self.gconfclient = SafeGConfClient()
     init_proxy(self.gconfclient)
     # init show version
-    try:
-        self.show_versions = self.gconfclient.get_bool("/apps/update-manager/show_versions")
-    except gobject.GError, e:
-        self.show_versions = False
+    self.show_versions = self.gconfclient.get_bool("/apps/update-manager/show_versions")
     # keep track when we run (for update-notifier)
-    try:
-        self.gconfclient.set_int("/apps/update-manager/launch_time", int(time.time()))
-    except gobject.GError, e:
-        print "Error setting launch_time: ", e
+    self.gconfclient.set_int("/apps/update-manager/launch_time", int(time.time()))
+
     # get progress object
     self.progress = GtkProgress.GtkOpProgress(self.dialog_cacheprogress,
                                               self.progressbar_cache,
@@ -235,6 +233,7 @@ class UpdateManager(SimpleGtkbuilderApp):
       """check and warn if on battery"""
       if on_battery():
           self.dialog_on_battery.set_transient_for(self.window_main)
+          self.dialog_on_battery.set_title("")
           res = self.dialog_on_battery.run()
           self.dialog_on_battery.hide()
           if res != gtk.RESPONSE_YES:
@@ -246,8 +245,13 @@ class UpdateManager(SimpleGtkbuilderApp):
     renderer.set_property("visible", pkg != None)
     if pkg is None:
         return
+    current_state = renderer.get_property("active")
     to_install = pkg.markedInstall or pkg.markedUpgrade
     renderer.set_property("active", to_install)
+    # we need to update the store as well to ensure orca knowns
+    # about state changes (it will not read view_func changes)
+    if to_install != current_state:
+        self.store[iter][LIST_TOGGLE_CHECKED] = to_install
     if pkg.name in self.list.held_back:
         renderer.set_property("activatable", False)
     else: 
@@ -448,8 +452,8 @@ class UpdateManager(SimpleGtkbuilderApp):
       try:
           inst_count = self.cache.installCount
           self.dl_size = self.cache.requiredDownload
-          t = _("%s selected. Download size: %s") % (inst_count,
-                                                       humanize_size(self.dl_size))
+          t = _("Download size: %s\n%s selected.") % (
+                                                      humanize_size(self.dl_size),inst_count)
           self.label_downsize.set_text(t)
       except SystemError, e:
           print "requiredDownload could not be calculated: %s" % e
@@ -589,6 +593,22 @@ class UpdateManager(SimpleGtkbuilderApp):
         return
     self.invoke_manager(INSTALL)
     
+  def show_reboot_required_dialog(self):
+    self.dialog_reboot.set_transient_for(self.window_main)
+    self.dialog_reboot.set_title("")
+    res = self.dialog_reboot.run()
+    self.dialog_reboot.hide()
+    if res == gtk.RESPONSE_OK:
+      try:
+          bus = dbus.SessionBus()
+          proxy_obj = bus.get_object("org.gnome.SessionManager",
+				     "/org/gnome/SessionManager")
+          iface = dbus.Interface(proxy_obj, "org.gnome.SessionManager")
+          iface.RequestReboot()
+          # FIXME: try sesion restart with hal?
+      except dbus.DBusException, e:
+          pass
+
   def invoke_manager(self, action):
     # check first if no other package manager is runing
 
@@ -606,8 +626,12 @@ class UpdateManager(SimpleGtkbuilderApp):
     if action == UPDATE:
         self.install_backend.update()
     elif action == INSTALL:
+        has_reboot = os.path.exists(REBOOT_REQUIRED_FILE)
+        # do it
         self.install_backend.commit(self.cache)
-
+        # check if there is a new reboot required notification
+        if not has_reboot and os.path.exists(REBOOT_REQUIRED_FILE):
+            self.show_reboot_required_dialog()
     s = _("Reading package information")
     self.label_cache_progress_title.set_label("<b><big>%s</big></b>" % s)
     self.fillstore()
@@ -695,12 +719,8 @@ class UpdateManager(SimpleGtkbuilderApp):
   def save_state(self):
     """ save the state  (window-size for now) """
     (x,y) = self.window_main.get_size()
-    try:
-        self.gconfclient.set_pair("/apps/update-manager/window_size",
-                                  gconf.VALUE_INT, gconf.VALUE_INT, x, y)
-    except gobject.GError, e:
-        print "Could not save the configuration to gconf: %s" % e
-        pass
+    self.gconfclient.set_pair("/apps/update-manager/window_size",
+                              gconf.VALUE_INT, gconf.VALUE_INT, x, y)
 
   def restore_state(self):
     """ restore the state (window-size for now) """
@@ -767,7 +787,7 @@ class UpdateManager(SimpleGtkbuilderApp):
       origin_list.reverse()
       for origin in origin_list:
         self.store.append(['<b><big>%s</big></b>' % origin.description,
-                           origin.description, None, origin])
+                           origin.description, None, origin,True])
         for pkg in self.list.pkgs[origin]:
           name = xml.sax.saxutils.escape(pkg.name)
           if not pkg.isInstalled:
@@ -786,7 +806,7 @@ class UpdateManager(SimpleGtkbuilderApp):
               contents = "%s\n<small>%s %s</small>" % (contents, version, size)
           else:
               contents = "%s <small>%s</small>" % (contents, size)
-          self.store.append([contents, pkg.name, pkg, None])
+          self.store.append([contents, pkg.name, pkg, None, True])
     self.update_count()
     self.setBusy(False)
     self.check_all_updates_installable()
@@ -822,7 +842,7 @@ class UpdateManager(SimpleGtkbuilderApp):
 
   def on_button_dist_upgrade_clicked(self, button):
       #print "on_button_dist_upgrade_clicked"
-      fetcher = DistUpgradeFetcherGtk(new_dist=self.new_dist, parent=self, progress=GtkProgress.GtkFetchProgress(self))
+      fetcher = DistUpgradeFetcherGtk(new_dist=self.new_dist, parent=self, progress=GtkProgress.GtkFetchProgress(self, _("Downloading the release upgrade tool")))
       if self.options.sandbox:
           fetcher.run_options.append("--sandbox")
       fetcher.run()
@@ -894,6 +914,7 @@ class UpdateManager(SimpleGtkbuilderApp):
       update_days = apt_pkg.Config.FindI("APT::Periodic::Update-Package-Lists")
       if update_days < 1:
           self.dialog_manual_update.set_transient_for(self.window_main)
+          self.dialog_manual_update.set_title("")
           res = self.dialog_manual_update.run()
           self.dialog_manual_update.hide()
           if res == gtk.RESPONSE_YES:
@@ -907,6 +928,7 @@ class UpdateManager(SimpleGtkbuilderApp):
 
   def ask_run_partial_upgrade(self):
       self.dialog_dist_upgrade.set_transient_for(self.window_main)
+      self.dialog_dist_upgrade.set_title("")
       res = self.dialog_dist_upgrade.run()
       self.dialog_dist_upgrade.hide()
       if res == gtk.RESPONSE_YES:
@@ -918,7 +940,7 @@ class UpdateManager(SimpleGtkbuilderApp):
 
   def check_metarelease(self):
       " check for new meta-release information "
-      gconfclient = gconf.client_get_default()
+      gconfclient = SafeGConfClient()
       self.meta = MetaRelease(self.options.devel_release,
                               self.options.use_proposed)
       self.meta.connect("dist_no_longer_supported",self.dist_no_longer_supported)
